@@ -50,6 +50,7 @@ pub use lease::*;
 pub use ockam_core::compat;
 pub use ockam_core::println;
 pub use ockam_core::worker;
+pub use ockam_core::AsyncTryClone;
 pub use ockam_entity::*;
 pub use protocols::*;
 pub use remote_forwarder::*;
@@ -58,8 +59,8 @@ pub use unique::*;
 pub mod stream;
 
 pub use ockam_core::{
-    Address, Any, Encoded, Error, Message, ProtocolId, Result, Route, Routed, RouterMessage,
-    TransportMessage, Worker,
+    Address, Any, Encoded, Error, LocalMessage, Message, ProtocolId, Result, Route, Routed,
+    RouterMessage, TransportMessage, Worker,
 };
 
 pub use ockam_channel::SecureChannel;
